@@ -17,15 +17,20 @@
 #endif
 #include "input_event.h"
 #include "inputmethod.h"
+#include "wayland-client/linuxdmabuf.h"
+#include "wayland-client/viewporter.h"
+#include "wayland-linux-dmabuf-unstable-v1-client-protocol.h"
+#include "wayland-viewporter-client-protocol.h"
+#include "wayland-zkde-screencast-unstable-v1-client-protocol.h"
 #include "wayland/display.h"
 #include "wayland_server.h"
 #include "workspace.h"
-#include <wayland-zkde-screencast-unstable-v1-client-protocol.h>
 
 #include <KWayland/Client/appmenu.h>
 #include <KWayland/Client/compositor.h>
 #include <KWayland/Client/connection_thread.h>
 #include <KWayland/Client/datadevicemanager.h>
+#include <KWayland/Client/datasource.h>
 #include <KWayland/Client/event_queue.h>
 #include <KWayland/Client/output.h>
 #include <KWayland/Client/plasmashell.h>
@@ -189,7 +194,7 @@ XdgSurface *XdgPopup::xdgSurface() const
 
 void XdgPopup::xdg_popup_configure(int32_t x, int32_t y, int32_t width, int32_t height)
 {
-    Q_EMIT configureRequested(QRect(x, y, width, height));
+    Q_EMIT configureRequested(Rect(x, y, width, height));
 }
 
 void XdgPopup::xdg_popup_popup_done()
@@ -529,10 +534,8 @@ std::unique_ptr<Connection> Connection::setup(AdditionalWaylandInterfaces flags)
                 c->fifoManager = std::make_unique<FifoManagerV1>(*c->registry, name, version);
             }
         }
-        if (flags & AdditionalWaylandInterface::PresentationTime) {
-            if (interface == wp_presentation_interface.name) {
-                c->presentationTime = std::make_unique<PresentationTime>(*c->registry, name, version);
-            }
+        if (interface == wp_presentation_interface.name) {
+            c->presentationTime = std::make_unique<PresentationTime>(*c->registry, name, version);
         }
         if (flags & AdditionalWaylandInterface::XdgActivation) {
             if (interface == xdg_activation_v1_interface.name) {
@@ -557,6 +560,20 @@ std::unique_ptr<Connection> Connection::setup(AdditionalWaylandInterfaces flags)
                 c->primarySelectionManager = std::make_unique<WpPrimarySelectionDeviceManagerV1>(*c->registry, name, version);
             }
         }
+        if (flags & AdditionalWaylandInterface::XdgToplevelDragV1) {
+            if (interface == xdg_toplevel_drag_manager_v1_interface.name) {
+                c->toplevelDragManager = std::make_unique<XdgToplevelDragManagerV1>(*c->registry, name, version);
+            }
+        }
+        if (flags & AdditionalWaylandInterface::LinuxDmabuf && interface == zwp_linux_dmabuf_v1_interface.name) {
+            c->linuxDmabuf = std::make_unique<WaylandClient::LinuxDmabufV1>(*c->registry, name, version);
+        }
+        if (flags & AdditionalWaylandInterface::ColorRepresentation && interface == wp_color_representation_manager_v1_interface.name) {
+            c->colorRepresentation = std::make_unique<ColorRepresentationV1>(*c->registry, name, version);
+        }
+        if (flags & AdditionalWaylandInterface::Viewporter && interface == wp_viewporter_interface.name) {
+            c->viewporter = std::make_unique<WaylandClient::Viewporter>(*c->registry, name, 1u);
+        }
     });
 
     QSignalSpy allAnnounced(registry, &KWayland::Client::Registry::interfacesAnnounced);
@@ -700,6 +717,10 @@ Connection::~Connection()
     tabletManager.reset();
     keyState.reset();
     primarySelectionManager.reset();
+    toplevelDragManager.reset();
+    linuxDmabuf.reset();
+    colorRepresentation.reset();
+    viewporter.reset();
 
     delete queue; // Must be destroyed last
     queue = nullptr;
@@ -894,6 +915,26 @@ WpPrimarySelectionDeviceManagerV1 *primarySelectionManager()
     return s_waylandConnection->primarySelectionManager.get();
 }
 
+XdgToplevelDragManagerV1 *toplevelDragManager()
+{
+    return s_waylandConnection->toplevelDragManager.get();
+}
+
+WaylandClient::LinuxDmabufV1 *linuxDmabuf()
+{
+    return s_waylandConnection->linuxDmabuf.get();
+}
+
+ColorRepresentationV1 *colorRepresentation()
+{
+    return s_waylandConnection->colorRepresentation.get();
+}
+
+WaylandClient::Viewporter *viewporter()
+{
+    return s_waylandConnection->viewporter.get();
+}
+
 bool waitForWaylandSurface(Window *window)
 {
     if (window->surface()) {
@@ -1174,7 +1215,9 @@ std::unique_ptr<XdgToplevel> createXdgToplevelSurface(XdgShell *shell, KWayland:
     XdgSurface *xdgSurface = new XdgSurface(shell, surface);
     std::unique_ptr<XdgToplevel> xdgToplevel = std::make_unique<XdgToplevel>(xdgSurface);
 
-    setup(xdgToplevel.get());
+    if (setup) {
+        setup(xdgToplevel.get());
+    }
     waitForConfigured(xdgSurface);
 
     return xdgToplevel;
@@ -1353,7 +1396,7 @@ bool unlockScreen()
 }
 #endif // KWIN_BUILD_LOCKSCREEN
 
-bool renderNodeAvailable()
+static bool haveDrmNode(int type)
 {
 #if !HAVE_LIBDRM_FAUX
 #if defined(Q_OS_LINUX)
@@ -1377,11 +1420,21 @@ bool renderNodeAvailable()
         drmFreeDevices(devices.data(), devices.size());
     });
 
-    return std::any_of(devices.constBegin(), devices.constEnd(), [](drmDevice *device) {
-        return device->available_nodes & (1 << DRM_NODE_RENDER);
+    return std::any_of(devices.constBegin(), devices.constEnd(), [type](drmDevice *device) {
+        return device->available_nodes & (1 << type);
     });
 }
 
+bool renderNodeAvailable()
+{
+    return haveDrmNode(DRM_NODE_RENDER);
+}
+
+bool primaryNodeAvailable()
+{
+    return haveDrmNode(DRM_NODE_PRIMARY);
+}
+
 #if KWIN_BUILD_X11
 void XcbConnectionDeleter::operator()(xcb_connection_t *pointer)
 {
@@ -1868,6 +1921,26 @@ ColorManagerV1::~ColorManagerV1()
     wp_color_manager_v1_destroy(object());
 }
 
+ColorRepresentationV1::ColorRepresentationV1(::wl_registry *registry, uint32_t id, int version)
+    : QtWayland::wp_color_representation_manager_v1(registry, id, version)
+{
+}
+
+ColorRepresentationV1::~ColorRepresentationV1()
+{
+    destroy();
+}
+
+ColorRepresentationSurfaceV1::ColorRepresentationSurfaceV1(::wp_color_representation_surface_v1 *object)
+    : QtWayland::wp_color_representation_surface_v1(object)
+{
+}
+
+ColorRepresentationSurfaceV1::~ColorRepresentationSurfaceV1()
+{
+    destroy();
+}
+
 FifoManagerV1::FifoManagerV1(::wl_registry *registry, uint32_t id, int version)
     : QtWayland::wp_fifo_manager_v1(registry, id, version)
 {
@@ -2217,6 +2290,31 @@ std::unique_ptr<WpPrimarySelectionSourceV1> WpPrimarySelectionDeviceManagerV1::c
     return std::make_unique<WpPrimarySelectionSourceV1>(create_source());
 }
 
+XdgToplevelDragManagerV1::XdgToplevelDragManagerV1(::wl_registry *registry, uint32_t id, int version)
+    : QtWayland::xdg_toplevel_drag_manager_v1(registry, id, version)
+{
+}
+
+std::unique_ptr<XdgToplevelDragV1> XdgToplevelDragManagerV1::createDrag(KWayland::Client::DataSource *source)
+{
+    return std::make_unique<XdgToplevelDragV1>(get_xdg_toplevel_drag(*source));
+}
+
+XdgToplevelDragManagerV1::~XdgToplevelDragManagerV1()
+{
+    destroy();
+}
+
+XdgToplevelDragV1::XdgToplevelDragV1(::xdg_toplevel_drag_v1 *id)
+    : QtWayland::xdg_toplevel_drag_v1(id)
+{
+}
+
+XdgToplevelDragV1::~XdgToplevelDragV1()
+{
+    destroy();
+}
+
 void keyboardKeyPressed(quint32 key, quint32 time)
 {
     auto virtualKeyboard = static_cast<WaylandTestApplication *>(kwinApp())->virtualKeyboard();
@@ -2313,7 +2411,7 @@ void tabletPadDialEvent(double delta, int number, quint32 time)
     Q_EMIT virtualTabletPad->tabletPadDialEvent(number, delta, 0, std::chrono::milliseconds(time), virtualTabletPad);
 }
 
-void tabletPadRingEvent(int position, int number, quint32 group, quint32 mode, quint32 time)
+void tabletPadRingEvent(qreal position, int number, quint32 group, quint32 mode, quint32 time)
 {
     auto virtualTabletPad = static_cast<WaylandTestApplication *>(kwinApp())->virtualTabletPad();
     Q_EMIT virtualTabletPad->tabletPadRingEvent(number, position, true, group, mode, std::chrono::milliseconds(time), virtualTabletPad);
@@ -2353,6 +2451,91 @@ void tabletToolTipEvent(const QPointF &pos, qreal pressure, qreal xTilt, qreal y
     auto tool = static_cast<WaylandTestApplication *>(kwinApp())->virtualTabletTool();
     Q_EMIT tablet->tabletToolTipEvent(pos, pressure, xTilt, yTilt, rotation, distance, tipDown, sliderPosition, tool, std::chrono::milliseconds(time), tablet);
 }
+
+XdgToplevelWindow::XdgToplevelWindow(const std::function<void(XdgToplevel *toplevel)> &setup)
+    : m_surface(createSurface())
+    , m_toplevel(createXdgToplevelSurface(m_surface.get(), setup))
+{
+}
+
+XdgToplevelWindow::XdgToplevelWindow(const std::function<void(KWayland::Client::Surface *surface, XdgToplevel *toplevel)> &setup)
+    : m_surface(createSurface())
+    , m_toplevel(createXdgToplevelSurface(m_surface.get(), [this, &setup](XdgToplevel *toplevel) {
+        setup(m_surface.get(), toplevel);
+    }))
+{
+}
+
+XdgToplevelWindow::~XdgToplevelWindow()
+{
+    if (m_window) {
+        m_toplevel.reset();
+        m_surface.reset();
+        waitForWindowClosed(m_window);
+    }
+}
+
+bool XdgToplevelWindow::show(const QSize &size, const QColor &color)
+{
+    m_window = renderAndWaitForShown(m_surface.get(), size, color);
+    return m_window != nullptr;
+}
+
+bool XdgToplevelWindow::show(const QImage &image)
+{
+    m_window = renderAndWaitForShown(m_surface.get(), image);
+    return m_window != nullptr;
+}
+
+void XdgToplevelWindow::unmap()
+{
+    m_surface->attachBuffer((wl_buffer *)nullptr);
+    m_surface->commit(KWayland::Client::Surface::CommitFlag::None);
+    // unmapping destroys the KWin::Window
+    m_window = nullptr;
+}
+
+bool XdgToplevelWindow::unmapAndWaitForClosed()
+{
+    Window *window = m_window;
+    unmap();
+    return waitForWindowClosed(window);
+}
+
+bool XdgToplevelWindow::presentWait()
+{
+    const auto feedback = std::make_unique<Test::WpPresentationFeedback>(Test::presentationTime()->feedback(*m_surface));
+    m_surface->commit(KWayland::Client::Surface::CommitFlag::None);
+    QSignalSpy spy(feedback.get(), &Test::WpPresentationFeedback::presented);
+    return spy.wait();
+}
+
+bool XdgToplevelWindow::waitSurfaceConfigure()
+{
+    QSignalSpy surfaceConfigure(m_toplevel->xdgSurface(), &Test::XdgSurface::configureRequested);
+    return surfaceConfigure.wait();
+}
+
+std::optional<QSize> XdgToplevelWindow::handleConfigure(const QColor &color)
+{
+    QSignalSpy toplevelConfigure(m_toplevel.get(), &Test::XdgToplevel::configureRequested);
+    QSignalSpy surfaceConfigure(m_toplevel->xdgSurface(), &Test::XdgSurface::configureRequested);
+    if (!toplevelConfigure.wait()) {
+        return std::nullopt;
+    }
+    m_toplevel->xdgSurface()->ack_configure(surfaceConfigure.last().at(0).value<quint32>());
+    const QSize ret = toplevelConfigure.last().at(0).toSize();
+    if (ret == m_surface->size()) {
+        m_surface->commit(KWayland::Client::Surface::CommitFlag::None);
+        return ret;
+    }
+    Test::render(m_surface.get(), toplevelConfigure.last().at(0).toSize(), color);
+    QSignalSpy frameGeometryChanged(m_window, &KWin::Window::frameGeometryChanged);
+    if (!frameGeometryChanged.wait()) {
+        return std::nullopt;
+    }
+    return ret;
+}
 }
 }
 
