@@ -89,7 +89,7 @@ static QByteArray readWindowProperty(xcb_window_t win, xcb_atom_t atom, xcb_atom
             len *= 2;
             continue;
         }
-        return prop.toByteArray(format, type);
+        return prop.toByteArray(format, type).value_or(QByteArray());
     }
 }
 
@@ -367,13 +367,13 @@ void EffectsHandler::prePaintScreen(ScreenPrePaintData &data, std::chrono::milli
     // no special final code
 }
 
-void EffectsHandler::paintScreen(const RenderTarget &renderTarget, const RenderViewport &viewport, int mask, const QRegion &region, Output *screen)
+void EffectsHandler::paintScreen(const RenderTarget &renderTarget, const RenderViewport &viewport, int mask, const Region &deviceRegion, LogicalOutput *screen)
 {
     if (m_currentPaintScreenIterator != m_activeEffects.constEnd()) {
-        (*m_currentPaintScreenIterator++)->paintScreen(renderTarget, viewport, mask, region, screen);
+        (*m_currentPaintScreenIterator++)->paintScreen(renderTarget, viewport, mask, deviceRegion, screen);
         --m_currentPaintScreenIterator;
     } else {
-        m_scene->finalPaintScreen(renderTarget, viewport, mask, region, screen);
+        m_scene->finalPaintScreen(renderTarget, viewport, mask, deviceRegion, screen);
     }
 }
 
@@ -386,34 +386,25 @@ void EffectsHandler::postPaintScreen()
     // no special final code
 }
 
-void EffectsHandler::prePaintWindow(EffectWindow *w, WindowPrePaintData &data, std::chrono::milliseconds presentTime)
+void EffectsHandler::prePaintWindow(RenderView *view, EffectWindow *w, WindowPrePaintData &data, std::chrono::milliseconds presentTime)
 {
     if (m_currentPaintWindowIterator != m_activeEffects.constEnd()) {
-        (*m_currentPaintWindowIterator++)->prePaintWindow(w, data, presentTime);
+        (*m_currentPaintWindowIterator++)->prePaintWindow(view, w, data, presentTime);
         --m_currentPaintWindowIterator;
     }
     // no special final code
 }
 
-void EffectsHandler::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const QRegion &region, WindowPaintData &data)
+void EffectsHandler::paintWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const Region &deviceRegion, WindowPaintData &data)
 {
     if (m_currentPaintWindowIterator != m_activeEffects.constEnd()) {
-        (*m_currentPaintWindowIterator++)->paintWindow(renderTarget, viewport, w, mask, region, data);
+        (*m_currentPaintWindowIterator++)->paintWindow(renderTarget, viewport, w, mask, deviceRegion, data);
         --m_currentPaintWindowIterator;
     } else {
-        m_scene->finalPaintWindow(renderTarget, viewport, w, mask, region, data);
+        m_scene->finalPaintWindow(renderTarget, viewport, w, mask, deviceRegion, data);
     }
 }
 
-void EffectsHandler::postPaintWindow(EffectWindow *w)
-{
-    if (m_currentPaintWindowIterator != m_activeEffects.constEnd()) {
-        (*m_currentPaintWindowIterator++)->postPaintWindow(w);
-        --m_currentPaintWindowIterator;
-    }
-    // no special final code
-}
-
 Effect *EffectsHandler::provides(Effect::Feature ef)
 {
     for (int i = 0; i < loaded_effects.size(); ++i) {
@@ -424,19 +415,19 @@ Effect *EffectsHandler::provides(Effect::Feature ef)
     return nullptr;
 }
 
-void EffectsHandler::drawWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const QRegion &region, WindowPaintData &data)
+void EffectsHandler::drawWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const Region &deviceRegion, WindowPaintData &data)
 {
     if (m_currentDrawWindowIterator != m_activeEffects.constEnd()) {
-        (*m_currentDrawWindowIterator++)->drawWindow(renderTarget, viewport, w, mask, region, data);
+        (*m_currentDrawWindowIterator++)->drawWindow(renderTarget, viewport, w, mask, deviceRegion, data);
         --m_currentDrawWindowIterator;
     } else {
-        m_scene->finalDrawWindow(renderTarget, viewport, w, mask, region, data);
+        m_scene->finalDrawWindow(renderTarget, viewport, w, mask, deviceRegion, data);
     }
 }
 
-void EffectsHandler::renderWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const QRegion &region, WindowPaintData &data)
+void EffectsHandler::renderWindow(const RenderTarget &renderTarget, const RenderViewport &viewport, EffectWindow *w, int mask, const Region &deviceRegion, WindowPaintData &data)
 {
-    m_scene->finalDrawWindow(renderTarget, viewport, w, mask, region, data);
+    m_scene->finalDrawWindow(renderTarget, viewport, w, mask, deviceRegion, data);
 }
 
 bool EffectsHandler::hasDecorationShadows() const
@@ -653,7 +644,7 @@ bool EffectsHandler::tabletPadButtonEvent(uint button, bool pressed, std::chrono
     return false;
 }
 
-bool EffectsHandler::tabletPadStripEvent(int number, int position, bool isFinger, std::chrono::microseconds time, InputDevice *device)
+bool EffectsHandler::tabletPadStripEvent(int number, qreal position, bool isFinger, std::chrono::microseconds time, InputDevice *device)
 {
     // TODO: reverse call order?
     for (auto it = loaded_effects.constBegin(); it != loaded_effects.constEnd(); ++it) {
@@ -664,7 +655,7 @@ bool EffectsHandler::tabletPadStripEvent(int number, int position, bool isFinger
     return false;
 }
 
-bool EffectsHandler::tabletPadRingEvent(int number, int position, bool isFinger, std::chrono::microseconds time, InputDevice *device)
+bool EffectsHandler::tabletPadRingEvent(int number, qreal position, bool isFinger, std::chrono::microseconds time, InputDevice *device)
 {
     // TODO: reverse call order?
     for (auto it = loaded_effects.constBegin(); it != loaded_effects.constEnd(); ++it) {
@@ -820,7 +811,7 @@ void EffectsHandler::windowToDesktops(EffectWindow *w, const QList<VirtualDeskto
     window->setDesktops(desktops);
 }
 
-void EffectsHandler::windowToScreen(EffectWindow *w, Output *screen)
+void EffectsHandler::windowToScreen(EffectWindow *w, LogicalOutput *screen)
 {
     auto window = w->window();
     if (window->isClient() && !window->isDesktop() && !window->isDock()) {
@@ -954,10 +945,8 @@ EffectWindow *EffectsHandler::findWindow(WId id) const
 }
 EffectWindow *EffectsHandler::findWindow(SurfaceInterface *surf) const
 {
-    if (waylandServer()) {
-        if (Window *w = waylandServer()->findWindow(surf)) {
-            return w->effectWindow();
-        }
+    if (Window *w = waylandServer()->findWindow(surf)) {
+        return w->effectWindow();
     }
     return nullptr;
 }
@@ -1064,32 +1053,42 @@ void EffectsHandler::addRepaintFull()
     m_compositor->scene()->addRepaintFull();
 }
 
-void EffectsHandler::addRepaint(const QRect &r)
+void EffectsHandler::addRepaint(const QRect &logicalRegion)
 {
-    m_compositor->scene()->addRepaint(r);
+    m_compositor->scene()->addLogicalRepaint(Rect(logicalRegion));
 }
 
-void EffectsHandler::addRepaint(const QRectF &r)
+void EffectsHandler::addRepaint(const QRectF &logicalRegion)
 {
-    m_compositor->scene()->addRepaint(r.toAlignedRect());
+    m_compositor->scene()->addLogicalRepaint(Rect(logicalRegion.toAlignedRect()));
 }
 
-void EffectsHandler::addRepaint(const QRegion &r)
+void EffectsHandler::addRepaint(const Rect &logicalRegion)
 {
-    m_compositor->scene()->addRepaint(r);
+    m_compositor->scene()->addLogicalRepaint(logicalRegion);
+}
+
+void EffectsHandler::addRepaint(const RectF &logicalRegion)
+{
+    m_compositor->scene()->addLogicalRepaint(logicalRegion.toAlignedRect());
+}
+
+void EffectsHandler::addRepaint(const Region &logicalRegion)
+{
+    m_compositor->scene()->addLogicalRepaint(logicalRegion);
 }
 
 void EffectsHandler::addRepaint(int x, int y, int w, int h)
 {
-    m_compositor->scene()->addRepaint(x, y, w, h);
+    m_compositor->scene()->addLogicalRepaint(x, y, w, h);
 }
 
-Output *EffectsHandler::activeScreen() const
+LogicalOutput *EffectsHandler::activeScreen() const
 {
     return workspace()->activeOutput();
 }
 
-QRectF EffectsHandler::clientArea(clientAreaOption opt, const Output *screen, const VirtualDesktop *desktop) const
+QRectF EffectsHandler::clientArea(clientAreaOption opt, const LogicalOutput *screen, const VirtualDesktop *desktop) const
 {
     return Workspace::self()->clientArea(opt, screen, desktop);
 }
@@ -1102,7 +1101,7 @@ QRectF EffectsHandler::clientArea(clientAreaOption opt, const EffectWindow *effe
 
 QRectF EffectsHandler::clientArea(clientAreaOption opt, const QPoint &p, const VirtualDesktop *desktop) const
 {
-    const Output *output = Workspace::self()->outputAt(p);
+    const LogicalOutput *output = Workspace::self()->outputAt(p);
     return Workspace::self()->clientArea(opt, output, desktop);
 }
 
@@ -1373,10 +1372,7 @@ bool EffectsHandler::blocksDirectScanout() const
 
 Display *EffectsHandler::waylandDisplay() const
 {
-    if (waylandServer()) {
-        return waylandServer()->display();
-    }
-    return nullptr;
+    return waylandServer()->display();
 }
 
 QVariant EffectsHandler::kwinOption(KWinOption kwopt)
@@ -1406,14 +1402,14 @@ QString EffectsHandler::supportInformation(const QString &name) const
         return QString();
     }
 
-    QString support((*it).first + QLatin1String(":\n"));
+    QString support((*it).first + QLatin1StringView(":\n"));
     const QMetaObject *metaOptions = (*it).second->metaObject();
     for (int i = 0; i < metaOptions->propertyCount(); ++i) {
         const QMetaProperty property = metaOptions->property(i);
         if (qstrcmp(property.name(), "objectName") == 0) {
             continue;
         }
-        support += QString::fromUtf8(property.name()) + QLatin1String(": ") + (*it).second->property(property.name()).toString() + QLatin1Char('\n');
+        support += QString::fromUtf8(property.name()) + QLatin1StringView(": ") + (*it).second->property(property.name()).toString() + QLatin1Char('\n');
     }
 
     return support;
@@ -1599,20 +1595,20 @@ SessionState EffectsHandler::sessionState() const
     return Workspace::self()->sessionManager()->state();
 }
 
-QList<Output *> EffectsHandler::screens() const
+QList<LogicalOutput *> EffectsHandler::screens() const
 {
     return Workspace::self()->outputs();
 }
 
-Output *EffectsHandler::screenAt(const QPoint &point) const
+LogicalOutput *EffectsHandler::screenAt(const QPoint &point) const
 {
     return Workspace::self()->outputAt(point);
 }
 
-Output *EffectsHandler::findScreen(const QString &name) const
+LogicalOutput *EffectsHandler::findScreen(const QString &name) const
 {
     const auto outputs = Workspace::self()->outputs();
-    for (Output *screen : outputs) {
+    for (LogicalOutput *screen : outputs) {
         if (screen->name() == name) {
             return screen;
         }
@@ -1620,7 +1616,7 @@ Output *EffectsHandler::findScreen(const QString &name) const
     return nullptr;
 }
 
-Output *EffectsHandler::findScreen(int screenId) const
+LogicalOutput *EffectsHandler::findScreen(int screenId) const
 {
     return Workspace::self()->outputs().value(screenId);
 }
@@ -1663,7 +1659,7 @@ QQmlEngine *EffectsHandler::qmlEngine() const
 
 void EffectsHandler::configChanged(const KConfigGroup &group, const QByteArrayList &names)
 {
-    if (group.name() != QLatin1String("Plugins")) {
+    if (group.name() != QLatin1StringView("Plugins")) {
         return;
     }
 
