@@ -53,9 +53,6 @@ namespace KWin
 // Mouse should not move more than this many pixels
 static const int DISTANCE_RESET = 30;
 
-// How large the touch target of the area recognizing touch gestures is
-static const int TOUCH_TARGET = 8;
-
 TouchCallback::TouchCallback(QAction *touchUpAction, TouchCallback::CallbackFunction progressCallback)
     : m_touchUpAction(touchUpAction)
     , m_progressCallback(progressCallback)
@@ -71,7 +68,7 @@ QAction *TouchCallback::touchUpAction() const
     return m_touchUpAction;
 }
 
-void TouchCallback::progressCallback(ElectricBorder border, const QPointF &deltaProgress, Output *output) const
+void TouchCallback::progressCallback(ElectricBorder border, const QPointF &deltaProgress, LogicalOutput *output) const
 {
     if (m_progressCallback) {
         m_progressCallback(border, deltaProgress, output);
@@ -94,7 +91,7 @@ Edge::Edge(ScreenEdges *parent)
     , m_pushBackBlocked(false)
     , m_client(nullptr)
     , m_output(nullptr)
-    , m_gesture(std::make_unique<ScreenEdgeGesture>(m_edges->gestureRecognizer(), SwipeDirection::Up, QRectF()))
+    , m_gesture(std::make_unique<ScreenEdgeGesture>(m_edges->gestureRecognizer(), SwipeDirection::Up, RectF()))
 {
     connect(
         m_gesture.get(), &ScreenEdgeGesture::triggered, this, [this]() {
@@ -548,7 +545,7 @@ void Edge::pushCursorBack(const QPoint &cursorPos)
     input()->pointer()->warp(QPoint(x, y));
 }
 
-void Edge::setGeometry(const QRect &geometry)
+void Edge::setGeometry(const Rect &geometry)
 {
     if (m_geometry == geometry) {
         return;
@@ -581,7 +578,7 @@ void Edge::setGeometry(const QRect &geometry)
             height = offset;
         }
     }
-    m_approachGeometry = QRect(x, y, width, height);
+    m_approachGeometry = Rect(x, y, width, height);
 
     if (isScreenEdge()) {
         m_gesture->setGeometry(m_geometry);
@@ -591,7 +588,7 @@ void Edge::setGeometry(const QRect &geometry)
 void Edge::checkBlocking()
 {
     Window *client = Workspace::self()->activeWindow();
-    const bool newValue = (!m_edges->remainActiveOnFullscreen() || m_client) && client && client->isFullScreen() && exclusiveContains(client->frameGeometry(), m_geometry.center()) && !(effects && effects->hasActiveFullScreenEffect());
+    const bool newValue = (!m_edges->remainActiveOnFullscreen() || m_client) && client && client->isFullScreen() && client->frameGeometry().contains(m_geometry.center()) && !(effects && effects->hasActiveFullScreenEffect());
     if (newValue == m_blocked) {
         return;
     }
@@ -639,7 +636,7 @@ void Edge::stopApproaching()
 
 void Edge::updateApproaching(const QPointF &point)
 {
-    if (exclusiveContains(approachGeometry(), point)) {
+    if (RectF(approachGeometry()).contains(point)) {
         int factor = 0;
         const int edgeDistance = m_edges->cornerOffset();
         auto cornerDistance = [=](const QPointF &corner) {
@@ -723,12 +720,12 @@ void Edge::setClient(Window *client)
     }
 }
 
-void Edge::setOutput(Output *output)
+void Edge::setOutput(LogicalOutput *output)
 {
     m_output = output;
 }
 
-Output *Edge::output() const
+LogicalOutput *Edge::output() const
 {
     return m_output;
 }
@@ -752,6 +749,7 @@ ScreenEdges::ScreenEdges()
     , m_actionBottomLeft(ElectricActionNone)
     , m_actionLeft(ElectricActionNone)
     , m_cornerOffset(40)
+    , m_touchTarget(8)
     , m_gestureRecognizer(std::make_unique<ScreenEdgeGestureRecognizer>())
 {
 }
@@ -773,11 +771,11 @@ static ElectricBorderAction electricBorderAction(const QString &name)
         return ElectricActionShowDesktop;
     } else if (lowerName == QLatin1StringView("lockscreen")) {
         return ElectricActionLockScreen;
-    } else if (lowerName == QLatin1String("krunner")) {
+    } else if (lowerName == QLatin1StringView("krunner")) {
         return ElectricActionKRunner;
-    } else if (lowerName == QLatin1String("activitymanager")) {
+    } else if (lowerName == QLatin1StringView("activitymanager")) {
         return ElectricActionActivityManager;
-    } else if (lowerName == QLatin1String("applicationlauncher")) {
+    } else if (lowerName == QLatin1StringView("applicationlauncher")) {
         return ElectricActionApplicationLauncher;
     }
     return ElectricActionNone;
@@ -790,6 +788,7 @@ void ScreenEdges::reconfigure()
     }
     KConfigGroup screenEdgesConfig = m_config->group(QStringLiteral("ScreenEdges"));
     setRemainActiveOnFullscreen(screenEdgesConfig.readEntry("RemainActiveOnFullscreen", false));
+    m_touchTarget = screenEdgesConfig.readEntry("TouchTarget", 8);
 
     // TODO: migrate settings to a group ScreenEdges
     KConfigGroup windowsConfig = m_config->group(QStringLiteral("Windows"));
@@ -808,6 +807,7 @@ void ScreenEdges::reconfigure()
     }
     const int pushBack = windowsConfig.readEntry("ElectricBorderPushbackPixels", 1);
     m_cursorPushBackDistance = QSize(pushBack, pushBack);
+    setAllScreenCorners(windowsConfig.readEntry("ElectricBorderAllScreenCorner", true));
 
     KConfigGroup borderConfig = m_config->group(QStringLiteral("ElectricBorders"));
     setActionForBorder(ElectricTopLeft, &m_actionTopLeft,
@@ -924,18 +924,18 @@ void ScreenEdges::updateLayout()
     }
 }
 
-static bool isLeftScreen(const QRect &screen, const QRect &fullArea)
+static bool isLeftScreen(const Rect &screen, const Rect &fullArea)
 {
     const auto outputs = workspace()->outputs();
     if (outputs.count() == 1) {
         return true;
     }
-    if (screen.x() == fullArea.x()) {
+    if (screen.left() == fullArea.left()) {
         return true;
     }
     // If any other screen has a right edge against our left edge, then this screen is not a left screen
-    for (const Output *output : outputs) {
-        const QRect otherGeo = output->geometry();
+    for (const LogicalOutput *output : outputs) {
+        const Rect otherGeo = output->geometry();
         if (otherGeo == screen) {
             // that's our screen to test
             continue;
@@ -951,18 +951,18 @@ static bool isLeftScreen(const QRect &screen, const QRect &fullArea)
     return true;
 }
 
-static bool isRightScreen(const QRect &screen, const QRect &fullArea)
+static bool isRightScreen(const Rect &screen, const Rect &fullArea)
 {
     const auto outputs = workspace()->outputs();
     if (outputs.count() == 1) {
         return true;
     }
-    if (screen.x() + screen.width() == fullArea.x() + fullArea.width()) {
+    if (screen.right() == fullArea.right()) {
         return true;
     }
     // If any other screen has any left edge against any of our right edge, then this screen is not a right screen
-    for (const Output *output : outputs) {
-        const QRect otherGeo = output->geometry();
+    for (const LogicalOutput *output : outputs) {
+        const Rect otherGeo = output->geometry();
         if (otherGeo == screen) {
             // that's our screen to test
             continue;
@@ -978,18 +978,18 @@ static bool isRightScreen(const QRect &screen, const QRect &fullArea)
     return true;
 }
 
-static bool isTopScreen(const QRect &screen, const QRect &fullArea)
+static bool isTopScreen(const Rect &screen, const Rect &fullArea)
 {
     const auto outputs = workspace()->outputs();
     if (outputs.count() == 1) {
         return true;
     }
-    if (screen.y() == fullArea.y()) {
+    if (screen.top() == fullArea.top()) {
         return true;
     }
     // If any other screen has any bottom edge against any of our top edge, then this screen is not a top screen
-    for (const Output *output : outputs) {
-        const QRect otherGeo = output->geometry();
+    for (const LogicalOutput *output : outputs) {
+        const Rect otherGeo = output->geometry();
         if (otherGeo == screen) {
             // that's our screen to test
             continue;
@@ -1005,18 +1005,18 @@ static bool isTopScreen(const QRect &screen, const QRect &fullArea)
     return true;
 }
 
-static bool isBottomScreen(const QRect &screen, const QRect &fullArea)
+static bool isBottomScreen(const Rect &screen, const Rect &fullArea)
 {
     const auto outputs = workspace()->outputs();
     if (outputs.count() == 1) {
         return true;
     }
-    if (screen.y() + screen.height() == fullArea.y() + fullArea.height()) {
+    if (screen.bottom() == fullArea.bottom()) {
         return true;
     }
     // If any other screen has any top edge against any of our bottom edge, then this screen is not a bottom screen
-    for (const Output *output : outputs) {
-        const QRect otherGeo = output->geometry();
+    for (const LogicalOutput *output : outputs) {
+        const Rect otherGeo = output->geometry();
         if (otherGeo == screen) {
             // that's our screen to test
             continue;
@@ -1041,14 +1041,14 @@ void ScreenEdges::recreateEdges()
 {
     std::vector<std::unique_ptr<Edge>> oldEdges = std::move(m_edges);
     m_edges.clear();
-    const QRect fullArea = workspace()->geometry();
-    QRegion processedRegion;
+    const Rect fullArea = workspace()->geometry();
+    Region processedRegion;
 
     const auto outputs = workspace()->outputs();
-    for (Output *output : outputs) {
-        const QRegion screen = QRegion(output->geometry()).subtracted(processedRegion);
+    for (LogicalOutput *output : outputs) {
+        const Region screen = Region(output->geometry()).subtracted(processedRegion);
         processedRegion += screen;
-        for (const QRect &screenPart : screen) {
+        for (const Rect &screenPart : screen.rects()) {
             if (isLeftScreen(screenPart, fullArea)) {
                 // left most screen
                 createVerticalEdge(ElectricLeft, screenPart, fullArea, output);
@@ -1066,6 +1066,14 @@ void ScreenEdges::recreateEdges()
                 createHorizontalEdge(ElectricBottom, screenPart, fullArea, output);
             }
         }
+
+        if (isAllScreenCorners()) {
+            const Rect geo = output->geometry();
+            m_edges.push_back(createEdge(ElectricTopLeft, geo.x(), geo.y(), m_touchTarget, m_touchTarget, output));
+            m_edges.push_back(createEdge(ElectricTopRight, geo.x() + geo.width() - m_touchTarget, geo.y(), m_touchTarget, m_touchTarget, output));
+            m_edges.push_back(createEdge(ElectricBottomLeft, geo.x(), geo.y() + geo.height() - m_touchTarget, m_touchTarget, m_touchTarget, output));
+            m_edges.push_back(createEdge(ElectricBottomRight, geo.x() + geo.width() - m_touchTarget, geo.y() + geo.height() - m_touchTarget, m_touchTarget, m_touchTarget, output));
+        }
     }
     auto split = std::partition(oldEdges.begin(), oldEdges.end(), [](const auto &edge) {
         return !edge->client();
@@ -1094,49 +1102,53 @@ void ScreenEdges::recreateEdges()
     }
 }
 
-void ScreenEdges::createVerticalEdge(ElectricBorder border, const QRect &screen, const QRect &fullArea, Output *output)
+void ScreenEdges::createVerticalEdge(ElectricBorder border, const Rect &screen, const Rect &fullArea, LogicalOutput *output)
 {
     if (border != ElectricRight && border != KWin::ElectricLeft) {
         return;
     }
     int y = screen.y();
     int height = screen.height();
-    const int x = (border == ElectricLeft) ? screen.x() : screen.x() + screen.width() - TOUCH_TARGET;
-    if (isTopScreen(screen, fullArea)) {
+    const int x = (border == ElectricLeft) ? screen.left() : screen.right() - m_touchTarget;
+    if (!isAllScreenCorners() && isTopScreen(screen, fullArea)) {
         // also top most screen
         height -= m_cornerOffset;
         y += m_cornerOffset;
         // create top left/right edge
         const ElectricBorder edge = (border == ElectricLeft) ? ElectricTopLeft : ElectricTopRight;
-        m_edges.push_back(createEdge(edge, x, screen.y(), TOUCH_TARGET, TOUCH_TARGET, output));
+        m_edges.push_back(createEdge(edge, x, screen.y(), m_touchTarget, m_touchTarget, output));
     }
-    if (isBottomScreen(screen, fullArea)) {
+    if (!isAllScreenCorners() && isBottomScreen(screen, fullArea)) {
         // also bottom most screen
         height -= m_cornerOffset;
         // create bottom left/right edge
         const ElectricBorder edge = (border == ElectricLeft) ? ElectricBottomLeft : ElectricBottomRight;
-        m_edges.push_back(createEdge(edge, x, screen.y() + screen.height() - TOUCH_TARGET, TOUCH_TARGET, TOUCH_TARGET, output));
+        m_edges.push_back(createEdge(edge, x, screen.y() + screen.height() - m_touchTarget, m_touchTarget, m_touchTarget, output));
     }
     if (height <= m_cornerOffset) {
         // An overlap with another output is near complete. We ignore this border.
         return;
     }
-    m_edges.push_back(createEdge(border, x, y, TOUCH_TARGET, height, output));
+    if (isAllScreenCorners()) {
+        height -= 2 * m_cornerOffset;
+        y += m_cornerOffset;
+    }
+    m_edges.push_back(createEdge(border, x, y, m_touchTarget, height, output));
 }
 
-void ScreenEdges::createHorizontalEdge(ElectricBorder border, const QRect &screen, const QRect &fullArea, Output *output)
+void ScreenEdges::createHorizontalEdge(ElectricBorder border, const Rect &screen, const Rect &fullArea, LogicalOutput *output)
 {
     if (border != ElectricTop && border != ElectricBottom) {
         return;
     }
     int x = screen.x();
     int width = screen.width();
-    if (isLeftScreen(screen, fullArea)) {
+    if (!isAllScreenCorners() && isLeftScreen(screen, fullArea)) {
         // also left most - adjust only x and width
         x += m_cornerOffset;
         width -= m_cornerOffset;
     }
-    if (isRightScreen(screen, fullArea)) {
+    if (!isAllScreenCorners() && isRightScreen(screen, fullArea)) {
         // also right most edge
         width -= m_cornerOffset;
     }
@@ -1144,11 +1156,15 @@ void ScreenEdges::createHorizontalEdge(ElectricBorder border, const QRect &scree
         // An overlap with another output is near complete. We ignore this border.
         return;
     }
-    const int y = (border == ElectricTop) ? screen.y() : screen.y() + screen.height() - TOUCH_TARGET;
-    m_edges.push_back(createEdge(border, x, y, width, TOUCH_TARGET, output));
+    if (isAllScreenCorners()) {
+        width -= 2 * m_cornerOffset;
+        x += m_cornerOffset;
+    }
+    const int y = (border == ElectricTop) ? screen.top() : screen.bottom() - m_touchTarget;
+    m_edges.push_back(createEdge(border, x, y, width, m_touchTarget, output));
 }
 
-std::unique_ptr<Edge> ScreenEdges::createEdge(ElectricBorder border, int x, int y, int width, int height, Output *output, bool createAction)
+std::unique_ptr<Edge> ScreenEdges::createEdge(ElectricBorder border, int x, int y, int width, int height, LogicalOutput *output, bool createAction)
 {
     std::unique_ptr<Edge> edge = std::make_unique<Edge>(this);
     // Edges can not have negative size.
@@ -1156,7 +1172,7 @@ std::unique_ptr<Edge> ScreenEdges::createEdge(ElectricBorder border, int x, int
     Q_ASSERT(height >= 0);
 
     edge->setBorder(border);
-    edge->setGeometry(QRect(x, y, width, height));
+    edge->setGeometry(Rect(x, y, width, height));
     edge->setOutput(output);
     if (createAction) {
         const ElectricBorderAction action = actionForEdge(edge.get());
@@ -1313,10 +1329,10 @@ bool ScreenEdges::createEdgeForClient(Window *client, ElectricBorder border)
     int width = 0;
     int height = 0;
 
-    Output *output = client->output();
-    const QRect geo = client->frameGeometry().toRect();
+    LogicalOutput *output = client->output();
+    const Rect geo = client->frameGeometry().toRect();
 
-    const QRect screen = output->geometry();
+    const Rect screen = output->geometry();
     switch (border) {
     case ElectricTop:
         y = screen.y();
